package com.jiuqi.dna.launcher;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Element;
import java.io.FileInputStream;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.HttpURLConnection;
import org.eclipse.core.runtime.internal.adaptor.Locker_JavaIo;
import java.io.File;
import org.eclipse.core.runtime.internal.adaptor.Locker;

public class ServerDaemon implements Constants
{
    private static ServerDaemon instance;
    private Locker locker;
    
    public ServerDaemon() {
        this.locker = null;
    }
    
    public static ServerDaemon getInstance() {
        if (ServerDaemon.instance == null) {
            ServerDaemon.instance = new ServerDaemon();
        }
        return ServerDaemon.instance;
    }
    
    public static void startServer() throws Exception {
        getInstance().start();
    }
    
    public static void stopServer() throws Exception {
        getInstance().stop();
        System.exit(0);
    }
    
    public static void restartServer() throws Exception {
        getInstance().stop();
        getInstance().start();
    }
    
    private void start() throws Exception {
        final String filePath = String.valueOf(System.getProperty("com.jiuqi.dna.rootpath")) + "/work/pid";
        final File file = new File(filePath);
        if (this.locker == null) {
            this.locker = (Locker)new Locker_JavaIo(file);
        }
        if (this.locker.isLocked()) {
            throw new RuntimeException("Server is already running!");
        }
        this.locker.lock();
    }
    
    private void stop() throws Exception {
        HttpURLConnection conn = null;
        try {
            final URL url = this.getUrl();
            conn = (HttpURLConnection)url.openConnection();
            conn.addRequestProperty("action", "stop");
            conn.setDoOutput(true);
            conn.setRequestMethod("POST");
            final OutputStream out = conn.getOutputStream();
            out.write(1);
            out.close();
            final InputStream in = conn.getInputStream();
            while (in.read() != -1) {}
            if (this.locker != null) {
                this.locker.release();
            }
        }
        finally {
            if (conn != null) {
                conn.disconnect();
            }
        }
        if (conn != null) {
            conn.disconnect();
        }
    }
    
    private URL getUrl() throws Exception {
        final String path = String.valueOf(System.getProperty("com.jiuqi.dna.rootpath")) + "/work/dna-server.xml";
        String host = null;
        String port = null;
        InputStream input = null;
        try {
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            final DocumentBuilder builder = factory.newDocumentBuilder();
            input = new FileInputStream(path);
            final Document doc = builder.parse(input);
            final Element root = doc.getDocumentElement();
            final NodeList listenList = root.getElementsByTagName("listen");
            if (listenList != null && listenList.getLength() > 0) {
                final Node node = listenList.item(0);
                if (node.getNodeType() == 1) {
                    final Element elem = (Element)node;
                    host = elem.getAttribute("host");
                    port = elem.getAttribute("port");
                }
            }
        }
        finally {
            input.close();
        }
        input.close();
        if (host == null || "".equals(host)) {
            host = "localhost";
        }
        int portNum = -1;
        if (port != null && !"".equals(port)) {
            portNum = Integer.valueOf(port);
        }
        return new URL("http", host, portNum, "sys");
    }
}
